@@ -1,4 +1,4 @@
-# Copyright 2016-2021 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2016-2022 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -25,7 +25,7 @@ from reahl.browsertools.browsertools import XPath
 
 from reahl.sqlalchemysupport import Session
 from reahl.webdeclarative.webdeclarative import PersistedFile
-from reahl.component.modelinterface import exposed, Event, FileField, Action, ValidationConstraint
+from reahl.component.modelinterface import ExposedNames, Event, FileField, Action, ValidationConstraint
 from reahl.component.exceptions import DomainException
 from reahl.web.bootstrap.forms import Form
 from reahl.web.bootstrap.files import FileUploadInput, FileUploadPanel, Button, FormLayout, FileInput, FileInputButton
@@ -43,13 +43,11 @@ class FileInputButtonFixture(Fixture):
     def new_domain_object(self):
         class DomainObject:
             files = []
-            @exposed
-            def fields(self, fields):
-                fields.files = FileField(allow_multiple=True, label='Attached files')
+            fields = ExposedNames()
+            fields.files = lambda i: FileField(allow_multiple=True, label='Attached files')
 
-            @exposed
-            def events(self, events):
-                events.submit = Event(label='Submit')
+            events = ExposedNames()
+            events.submit = lambda i: Event(label='Submit')
 
         return DomainObject()
 
@@ -227,13 +225,11 @@ class FileUploadInputFixture(Fixture):
                 self.submitted_file_info = {}
                 self.submitted = False
                
-            @exposed
-            def fields(self, fields):
-                fields.files = FileField(allow_multiple=True, label='Attached files', required=True)
+            fields = ExposedNames()
+            fields.files = lambda i: FileField(allow_multiple=True, label='Attached files', required=True)
 
-            @exposed
-            def events(self, events):
-                events.submit = Event(label='Submit', action=Action(self.submit))
+            events = ExposedNames()
+            events.submit = lambda i: Event(label='Submit', action=Action(i.submit))
                 
             def submit(self):
                 if self.throws_exception:
@@ -275,13 +271,11 @@ class ConstrainedFileUploadInputFixture(FileUploadInputFixture):
     def new_domain_object(self):
         fixture = self
         class DomainObject:
-            @exposed
-            def fields(self, fields):
-                fields.files = fixture.file_field
+            fields = ExposedNames()
+            fields.files = lambda i: fixture.file_field
 
-            @exposed
-            def events(self, events):
-                events.submit = Event(label='Submit')
+            events = ExposedNames()
+            events.submit = lambda i: Event(label='Submit')
 
         return DomainObject()
 
@@ -329,13 +323,14 @@ class ToggleValidationFixture(FileUploadInputFixture):
     def new_domain_object(self):
         fixture = self
         class DomainObject:
-            @exposed
-            def fields(self, fields):
-                fields.files = FileField(allow_multiple=True, label='Attached files')
-                fields.files.add_validation_constraint(ToggleableConstraint(fixture))
-            @exposed
-            def events(self, events):
-                events.submit = Event(label='Submit')
+            fields = ExposedNames()
+            def make_field(self):
+                field = FileField(allow_multiple=True, label='Attached files')
+                field.add_validation_constraint(ToggleableConstraint(fixture))
+                return field
+            fields.files = make_field
+            events = ExposedNames()
+            events.submit = lambda i: Event(label='Submit')
                                 
         return DomainObject()
 
