@@ -1,4 +1,4 @@
-# Copyright 2013-2021 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2022 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -29,7 +29,7 @@ from reahl.browsertools.browsertools import WidgetTester, XPath, Browser
 
 from reahl.component.exceptions import DomainException, ProgrammerError, IsInstance
 from reahl.component.modelinterface import IntegerField, EmailField, DateField, \
-    exposed, Field, Event, Action, MultiChoiceField, Choice
+    ExposedNames, Field, Event, Action, MultiChoiceField, Choice
 from reahl.webdeclarative.webdeclarative import PersistedException, UserInput
 
 from reahl.sqlalchemysupport import Base, Session
@@ -54,9 +54,8 @@ def test_basic_event_linkup(web_fixture):
         def handle_event(self):
             self.handled_event = True
 
-        @exposed
-        def events(self, events):
-            events.an_event = Event(label='click me', action=Action(self.handle_event))
+        events = ExposedNames()
+        events.an_event = lambda i: Event(label='click me', action=Action(i.handle_event))
 
     model_object = ModelObject()
 
@@ -103,13 +102,11 @@ def test_button_submits_only_once(web_fixture):
         def clicked(self):
             fixture.click_count += 1
 
-        @exposed
-        def events(self, events):
-            events.an_event = Event(label='click me', action=Action(self.clicked))
+        events = ExposedNames()
+        events.an_event = lambda i: Event(label='click me', action=Action(i.clicked))
 
-        @exposed
-        def fields(self, fields):
-            fields.field_name = IntegerField()
+        fields = ExposedNames()
+        fields.field_name = lambda i: IntegerField()
 
     wsgi_app = web_fixture.new_wsgi_app(enable_js=True, child_factory=MyForm.factory())
     web_fixture.reahl_server.set_app(wsgi_app)
@@ -155,12 +152,11 @@ def test_arguments_to_actions(web_fixture):
         def handle_event(self, *args):
             self.args = args
 
-        @exposed
-        def events(self, events):
-            events.an_event = Event(label='click me',
-                                    action=Action(self.handle_event, ['one_argument', 'another_argument']),
-                                    one_argument=IntegerField(),
-                                    another_argument=Field())
+        events = ExposedNames()
+        events.an_event = lambda i: Event(label='click me',
+                                          action=Action(i.handle_event, ['one_argument', 'another_argument']),
+                                          one_argument=IntegerField(),
+                                          another_argument=Field())
 
     model_object = ModelObject()
 
@@ -193,9 +189,8 @@ def test_validation_of_event_arguments(web_fixture):
     fixture = web_fixture
 
     class ModelObject:
-        @exposed
-        def events(self, events):
-            events.an_event = Event(label='Click me', argument=Field(required=True))
+        events = ExposedNames()
+        events.an_event = lambda i: Event(label='Click me', argument=Field(required=True))
 
     model_object = ModelObject()
 
@@ -226,13 +221,11 @@ def test_basic_field_linkup(web_fixture):
         def handle_event(self):
             pass
 
-        @exposed
-        def events(self, events):
-            events.an_event = Event(label='click me', action=Action(self.handle_event))
+        events = ExposedNames()
+        events.an_event = lambda i: Event(label='click me', action=Action(i.handle_event))
 
-        @exposed
-        def fields(self, fields):
-            fields.field_name = IntegerField(default=3)
+        fields = ExposedNames()
+        fields.field_name = lambda i: IntegerField(default=3)
 
     model_object = ModelObject()
 
@@ -273,17 +266,15 @@ def test_distinguishing_identical_field_names(web_fixture):
     fixture = web_fixture
 
     class ModelObject:
-        @exposed
-        def fields(self, fields):
-            fields.field_name = IntegerField()
+        fields = ExposedNames()
+        fields.field_name = lambda i: IntegerField()
 
     model_object1 = ModelObject()
     model_object2 = ModelObject()
 
     class MyForm(Form):
-        @exposed
-        def events(self, events):
-            events.an_event = Event(label='click me')
+        events = ExposedNames()
+        events.an_event = lambda i: Event(label='click me')
 
         def __init__(self, view, name):
             super().__init__(view, name)
@@ -329,9 +320,8 @@ def test_define_event_handler_not_called(web_fixture):
     fixture = web_fixture
 
     class ModelObject:
-        @exposed
-        def events(self, events):
-            events.an_event = Event()
+        events = ExposedNames()
+        events.an_event = lambda i: Event()
 
     model_object = ModelObject()
 
@@ -365,12 +355,10 @@ def test_exception_handling(reahl_system_fixture, web_fixture, sql_alchemy_fixtu
         def handle_event(self):
             self.field_name = 1
             raise DomainException()
-        @exposed
-        def events(self, events):
-            events.an_event = Event(label='click me', action=Action(self.handle_event))
-        @exposed
-        def fields(self, fields):
-            fields.field_name = IntegerField(default=3)
+        events = ExposedNames()
+        events.an_event = lambda i: Event(label='click me', action=Action(i.handle_event))
+        fields = ExposedNames()
+        fields.field_name = lambda i: IntegerField(default=3)
 
     with sql_alchemy_fixture.persistent_test_classes(ModelObject):
         model_object = ModelObject()
@@ -421,16 +409,15 @@ def test_form_preserves_user_input_after_validation_exceptions_multichoice(web_f
     fixture = web_fixture
 
     class ModelObject:
-        @exposed
-        def events(self, events):
-            events.an_event = Event(label='click me')
-        @exposed
-        def fields(self, fields):
-            choices = [Choice(1, IntegerField(label='One')),
-                       Choice(2, IntegerField(label='Two')),
-                       Choice(3, IntegerField(label='Three'))]
-            fields.no_validation_exception_field = MultiChoiceField(choices, label='Make your invalid choice', default=[])
-            fields.validation_exception_field = MultiChoiceField(choices, label='Make your choice', default=[], required=True)
+        events = ExposedNames()
+        events.an_event = lambda i: Event(label='click me')
+            
+        choices = [Choice(1, IntegerField(label='One')),
+                   Choice(2, IntegerField(label='Two')),
+                   Choice(3, IntegerField(label='Three'))]
+        fields = ExposedNames()
+        fields.no_validation_exception_field = lambda i: MultiChoiceField(i.choices, label='Make your invalid choice', default=[])
+        fields.validation_exception_field = lambda i: MultiChoiceField(i.choices, label='Make your choice', default=[], required=True)
 
     model_object = ModelObject()
 
@@ -530,9 +517,8 @@ def test_check_missing_form(web_fixture):
     fixture = web_fixture
 
     class ModelObject:
-        @exposed
-        def fields(self, fields):
-            fields.name = Field()
+        fields = ExposedNames()
+        fields.name = lambda i: Field()
 
     class MyPanel(Div):
         def __init__(self, view):
@@ -564,12 +550,10 @@ def test_nested_forms(web_fixture):
         handled_event = False
         def handle_event(self):
             self.handled_event = True
-        @exposed
-        def events(self, events):
-            events.nested_event = Event(label='click nested', action=Action(self.handle_event))
-        @exposed
-        def fields(self, fields):
-            fields.nested_field = Field(label='input nested')
+        events = ExposedNames()
+        events.nested_event = lambda i: Event(label='click nested', action=Action(i.handle_event))
+        fields = ExposedNames()
+        fields.nested_field = lambda i: Field(label='input nested')
 
     nested_model_object = NestedModelObject()
     class MyNestedForm(NestedForm):
@@ -583,9 +567,8 @@ def test_nested_forms(web_fixture):
         handled_event = False
         def handle_event(self):
             self.handled_event = True
-        @exposed
-        def events(self, events):
-            events.outer_event = Event(label='click outer', action=Action(self.handle_event))
+        events = ExposedNames()
+        events.outer_event = lambda i: Event(label='click outer', action=Action(i.handle_event))
     outer_model_object = OuterModelObject()
     class OuterForm(Form):
         def __init__(self, view, name):
@@ -620,12 +603,10 @@ def test_form_input_validation(web_fixture):
     class ModelObject:
         def handle_event(self):
             pass
-        @exposed
-        def events(self, events):
-            events.an_event = Event(label='click me', action=Action(self.handle_event))
-        @exposed
-        def fields(self, fields):
-            fields.field_name = EmailField()
+        events = ExposedNames()
+        events.an_event = lambda i: Event(label='click me', action=Action(i.handle_event))
+        fields = ExposedNames()
+        fields.field_name = lambda i: EmailField()
 
     model_object = ModelObject()
 
@@ -719,7 +700,7 @@ class QueryStringScenarios(Fixture):
 
     def new_form(self):
         form = Form(self.web_fixture.view, 'some_form')
-        event = Event(label='click me', action=Action(self.action))
+        event = Event(label='click me', action=Action(i.action))
         event.bind('an_event', None)
         form.define_event_handler(event, target=self.target)
         form.add_child(ButtonInput(form, event))
@@ -766,9 +747,8 @@ def test_propagation_of_querystring(web_fixture, query_string_scenarios):
         def handle_event(self):
             if fixture.break_on_submit:
                 raise DomainException()
-        @exposed
-        def events(self, events):
-            events.an_event = Event(label='click me', action=Action(self.handle_event))
+        events = ExposedNames()
+        events.an_event = lambda i: Event(label='click me', action=Action(i.handle_event))
 
     model_object = ModelObject()
 
@@ -811,11 +791,10 @@ def test_event_names_are_canonicalised(web_fixture):
         def handle_event(self, some_argument):
             self.received_argument = some_argument
 
-        @exposed
-        def events(self, events):
-            events.an_event = Event(label='click me',
-                                    action=Action(self.handle_event, ['some_argument']),
-                                    some_argument=Field(default='default value'))
+        events = ExposedNames()
+        events.an_event = lambda i: Event(label='click me',
+                                          action=Action(i.handle_event, ['some_argument']),
+                                          some_argument=Field(default='default value'))
 
     model_object = ModelObject()
 
@@ -854,10 +833,9 @@ def test_alternative_event_trigerring(web_fixture):
         def handle_event(self):
             self.handled_event = True
 
-        @exposed
-        def events(self, events):
-            events.an_event = Event(label='click me',
-                                    action=Action(self.handle_event))
+        events = ExposedNames()
+        events.an_event = lambda i: Event(label='click me',
+                                    action=Action(i.handle_event))
 
     model_object = ModelObject()
 
@@ -901,9 +879,8 @@ def test_remote_field_validation(web_fixture):
     fixture = web_fixture
 
     class ModelObject:
-        @exposed
-        def fields(self, fields):
-            fields.a_field = EmailField()
+        fields = ExposedNames()
+        fields.a_field = lambda i: EmailField()
 
     model_object = ModelObject()
 
@@ -932,9 +909,8 @@ def test_remote_field_formatting(web_fixture):
     fixture = web_fixture
 
     class ModelObject:
-        @exposed
-        def fields(self, fields):
-            fields.a_field = DateField()
+        fields = ExposedNames()
+        fields.a_field = lambda i: DateField()
 
     model_object = ModelObject()
 
