@@ -1,4 +1,4 @@
-# Copyright 2013-2021 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2022 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -21,7 +21,7 @@ from reahl.stubble import stubclass
 
 from reahl.component.modelinterface import Field, ValidationConstraint, RequiredConstraint, MinLengthConstraint, \
                              MaxLengthConstraint, PatternConstraint, AllowedValuesConstraint, \
-                             EqualToConstraint, RemoteConstraint, exposed
+                             EqualToConstraint, RemoteConstraint, ExposedNames
 from reahl.web.ui import PrimitiveInput, HTMLInputElement, Form, TextInput, ButtonInput, FormLayout
 from reahl.browsertools.browsertools import WidgetTester, XPath
 
@@ -61,7 +61,6 @@ def test_rendering_of_constraints(web_fixture, constraint_rendering_fixture):
 
     fixture = constraint_rendering_fixture
 
-    fixture.field.bind('an_attribute', fixture.model_object)
     fixture.model_object.an_attribute = 'field value'
 
     constraint1 = ValidationConstraint('validation_constraint 1 message')
@@ -264,10 +263,9 @@ def test_equal_to_constraint_js(web_fixture, constraint_rendering_fixture):
     fixture = constraint_rendering_fixture
 
     class ModelObject:
-        @exposed
-        def fields(self, fields):
-            fields.an_attribute = Field(label='an attribute')
-            fields.other = Field(label='other attribute')
+        fields = ExposedNames()
+        fields.an_attribute = lambda i: Field(label='an attribute')
+        fields.other = lambda i: Field(label='other attribute')
 
     model_object = ModelObject()
     other_field = model_object.fields.other
