@@ -1,4 +1,4 @@
-# Copyright 2013-2021 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2022 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -23,7 +23,7 @@ from reahl.stubble import EmptyStub, stubclass
 from reahl.web.ui import HTMLElement, PrimitiveInput, Form, CheckboxInput, TextInput, Label, ButtonInput,\
                           PasswordInput, TextArea, SelectInput, RadioButtonSelectInput, CheckboxSelectInput
 
-from reahl.component.modelinterface import Field, EmailField, BooleanField, Event, Allowed, exposed, \
+from reahl.component.modelinterface import Field, EmailField, BooleanField, Event, Allowed, ExposedNames, \
     Action, Choice, ChoiceGroup, ChoiceField, IntegerField, MultiChoiceField, DateField
 from reahl.component.exceptions import IsInstance, ProgrammerError
 from reahl.browsertools.browsertools import WidgetTester
@@ -60,12 +60,10 @@ class SimpleInputFixture2(SimpleInputFixture):
         class ModelObject:
             def handle_event(self):
                 pass
-            @exposed
-            def events(self, events):
-                events.an_event = Event(label='click me', action=Action(self.handle_event))
-            @exposed
-            def fields(self, fields):
-                fields.an_attribute = fixture.field
+            events = ExposedNames()
+            events.an_event = lambda i: Event(label='click me', action=Action(i.handle_event))
+            fields = ExposedNames()
+            fields.an_attribute = lambda i: fixture.field
         return ModelObject()
 
     def new_Form(self, input_widget_class=None):
@@ -185,6 +183,14 @@ class InputScenarios(SimpleInputFixture):
         self.expected_html = '<input name="test-an_attribute" id="id-test-an_attribute" aria-label="some text" form="test" placeholder="some text" type="text" value="field value" class="reahl-primitiveinput reahl-textinput">'
         self.bound_field = self.widget.bound_field
 
+    @scenario
+    def text_input_placeholder_from_field_default(self):
+        self.field_fixture.model_object.an_attribute = None
+        field_with_default = self.new_field(default='my field default')
+        self.widget = TextInput(self.form, field_with_default, placeholder=True)
+        self.expected_html = '<input name="test-an_attribute" id="id-test-an_attribute" aria-label="my field default" form="test" placeholder="my field default" type="text" value="" class="reahl-primitiveinput reahl-textinput">'
+        self.bound_field = self.widget.bound_field
+
     @scenario
     def input_label(self):
         html_input = TextInput(self.form, self.field)
@@ -268,9 +274,9 @@ class InputScenarios(SimpleInputFixture):
         self.field = MultiChoiceField(choices)
         self.field.bind('an_attribute', self.model_object)
 
-        self.widget = self.form.add_child(SelectInput(self.form, self.field))
+        self.widget = self.form.add_child(SelectInput(self.form, self.field, size=2))
         options = '<option id="id-test-an_attribute-91--93--1" value="1">One</option><option id="id-test-an_attribute-91--93--2" selected="selected" value="2">Two</option>'
-        self.expected_html = '<select name="test-an_attribute[]" id="id-test-an_attribute-91--93-" form="test" multiple="multiple" class="reahl-primitiveinput">%s</select>' % (options)
+        self.expected_html = '<select name="test-an_attribute[]" id="id-test-an_attribute-91--93-" form="test" multiple="multiple" size="2" class="reahl-primitiveinput">%s</select>' % (options)
         self.bound_field = self.widget.bound_field
 
     @scenario
@@ -290,7 +296,7 @@ class InputScenarios(SimpleInputFixture):
         checked_options = r'<label><input name="test-an_attribute[]" id="id-test-an_attribute-91--93--2" checked="checked" form="test" type="checkbox" value="2">Two</label>'
         checked_options += r'<label><input name="test-an_attribute[]" id="id-test-an_attribute-91--93--3" checked="checked" form="test" type="checkbox" value="3">Three</label>'
         options = not_checked_option + checked_options
-        self.expected_html = r'<div class="reahl-checkbox-input reahl-primitiveinput">%s</div>' % (options)
+        self.expected_html = r'<div id="id-test-an_attribute-91--93-" class="reahl-checkbox-input reahl-primitiveinput">%s</div>' % (options)
         self.bound_field = self.widget.bound_field
 
     @scenario
@@ -313,7 +319,7 @@ class InputScenarios(SimpleInputFixture):
                               'type="radio" value="%s">%s'\
                        '</label>'
 
-        outer_div = '<div class="reahl-primitiveinput reahl-radio-button-input">%s</div>'
+        outer_div = '<div id="id-test-an_attribute" class="reahl-primitiveinput reahl-radio-button-input">%s</div>'
         buttons = (radio_button % ('1', '', '', '1', 'One')) +\
                   (radio_button % ('2', ' checked="checked"', '', '2', 'Two'))
         self.expected_html = outer_div % buttons
