@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2018-2022 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -25,10 +25,10 @@ from reahl.tofu.pytestsupport import with_fixtures
 
 from reahl.web_dev.fixtures import WebFixture
 from reahl.browsertools.browsertools import XPath
-from reahl.web.fw import Widget
+from reahl.web.fw import Widget, Url
 from reahl.web.ui import Form, Div, SelectInput, Label, P, RadioButtonSelectInput, CheckboxSelectInput, \
-    CheckboxInput, ButtonInput, TextInput, FormLayout
-from reahl.component.modelinterface import Field, BooleanField, MultiChoiceField, ChoiceField, Choice, exposed, \
+    CheckboxInput, ButtonInput, TextInput, FormLayout, A
+from reahl.component.modelinterface import Field, BooleanField, MultiChoiceField, ChoiceField, Choice, ExposedNames, \
     IntegerField, EmailField, Event, Action, Allowed
 from reahl.component.exceptions import DomainException
 from reahl.web_dev.inputandvalidation.test_widgetqueryargs import QueryStringFixture
@@ -44,12 +44,11 @@ class ResponsiveDisclosureFixture(Fixture):
             def __init__(self):
                 self.choice = 1
 
-            @exposed
-            def fields(self, fields):
-                fields.choice = ChoiceField([Choice(1, IntegerField(label='One')),
-                                             Choice(2, IntegerField(label='Two')),
-                                             Choice(3, IntegerField(label='Three'))],
-                                            label='Choice')
+            fields = ExposedNames()
+            fields.choice = lambda i: ChoiceField([Choice(1, IntegerField(label='One')),
+                                                   Choice(2, IntegerField(label='Two')),
+                                                   Choice(3, IntegerField(label='Three'))],
+                                                  label='Choice')
         return ModelObject
 
     def new_MainWidget(self):
@@ -107,10 +106,9 @@ class ResponsiveWidgetScenarios(ResponsiveDisclosureFixture):
         class ModelObject:
             def __init__(self):
                 self.choice = False
-            @exposed
-            def fields(self, fields):
-                fields.choice = BooleanField(label=u'Choice',
-                                             true_value=u'true_value', false_value=u'false_value')
+            fields = ExposedNames()
+            fields.choice = lambda i: BooleanField(label=u'Choice',
+                                                   true_value=u'true_value', false_value=u'false_value')
         self.ModelObject = ModelObject
 
         def create_trigger_input(form, an_object):
@@ -132,12 +130,11 @@ class ResponsiveWidgetScenarios(ResponsiveDisclosureFixture):
             def __init__(self):
                 self.choice = [1]
 
-            @exposed
-            def fields(self, fields):
-                fields.choice = MultiChoiceField([Choice(1, IntegerField(label='One')),
-                                                  Choice(2, IntegerField(label='Two')),
-                                                  Choice(3, IntegerField(label='Three'))],
-                                                 label='Choice')
+            fields = ExposedNames()
+            fields.choice = lambda i: MultiChoiceField([Choice(1, IntegerField(label='One')),
+                                                        Choice(2, IntegerField(label='Two')),
+                                                        Choice(3, IntegerField(label='Three'))],
+                                                       label='Choice')
         self.ModelObject = ModelObject
 
         def create_trigger_input(form, an_object):
@@ -160,10 +157,9 @@ class ResponsiveWidgetScenarios(ResponsiveDisclosureFixture):
             def __init__(self):
                 self.choice = [1]
 
-            @exposed
-            def fields(self, fields):
-                fields.choice = MultiChoiceField([Choice(1, IntegerField(label='One'))],
-                                                 label='Choice')
+            fields = ExposedNames()
+            fields.choice = lambda i: MultiChoiceField([Choice(1, IntegerField(label='One'))],
+                                                       label='Choice')
         self.ModelObject = ModelObject
 
         self.expected_focussed_element = XPath.input_labelled('One')
@@ -180,10 +176,9 @@ class ResponsiveWidgetScenarios(ResponsiveDisclosureFixture):
             def __init__(self):
                 self.choice = []
 
-            @exposed
-            def fields(self, fields):
-                fields.choice = MultiChoiceField([Choice(1, IntegerField(label='One'))],
-                                                 label='Choice')
+            fields = ExposedNames()
+            fields.choice = lambda i: MultiChoiceField([Choice(1, IntegerField(label='One'))],
+                                                       label='Choice')
         self.ModelObject = ModelObject
         #self.MyForm from multi_valued_checkbox_select
 
@@ -200,12 +195,11 @@ class ResponsiveWidgetScenarios(ResponsiveDisclosureFixture):
             def __init__(self):
                 self.choice = [1]
 
-            @exposed
-            def fields(self, fields):
-                fields.choice = MultiChoiceField([Choice(1, IntegerField(label='One')),
-                                                  Choice(2, IntegerField(label='Two')),
-                                                  Choice(3, IntegerField(label='Three'))],
-                                                 label='Choice')
+            fields = ExposedNames()
+            fields.choice = lambda i: MultiChoiceField([Choice(1, IntegerField(label='One')),
+                                                        Choice(2, IntegerField(label='Two')),
+                                                        Choice(3, IntegerField(label='Three'))],
+                                                       label='Choice')
         self.ModelObject = ModelObject
 
         def create_trigger_input(form, an_object):
@@ -240,8 +234,7 @@ def test_inputs_can_refresh_parent_widgets(web_fixture, query_string_fixture, re
 
 @with_fixtures(WebFixture, QueryStringFixture, ResponsiveDisclosureFixture)
 def test_refresh_input_can_be_set_after_input_is_created(web_fixture, query_string_fixture, responsive_disclosure_fixture):
-    """If the widget to be refreshed can only be created after the triggering Input, `set_refresh_widget` can be called
-       after construction of the Input to instruct it to refresh a given Widget."""
+    """`set_refresh_widgets` can be called after construction of the Input to instruct it to refresh a given list of Widgets."""
     fixture = responsive_disclosure_fixture
 
     class MyForm(Form):
@@ -252,7 +245,12 @@ def test_refresh_input_can_be_set_after_input_is_created(web_fixture, query_stri
             refreshed_widget = self.add_child(P(self.view, text='My state is now %s' % an_object.choice))
             refreshed_widget.set_id('refreshing-paragraph')
             refreshed_widget.enable_refresh()
-            trigger_input.set_refresh_widget(refreshed_widget)
+            
+            another_widget_to_refresh = self.add_child(P(self.view, text='Another widget state is now %s' % an_object.choice))
+            another_widget_to_refresh.set_id('another-refreshing-paragraph')
+            another_widget_to_refresh.enable_refresh()
+            
+            trigger_input.set_refresh_widgets([refreshed_widget, another_widget_to_refresh])
 
     fixture.MyForm = MyForm
 
@@ -262,8 +260,51 @@ def test_refresh_input_can_be_set_after_input_is_created(web_fixture, query_stri
     browser.open('/')
 
     assert browser.wait_for(query_string_fixture.is_state_now, 1)
+    assert browser.wait_for(query_string_fixture.is_state_on_another_widget_now, 1)
     browser.select(XPath.select_labelled('Choice'), 'Three')
     assert browser.wait_for(query_string_fixture.is_state_now, 3)
+    assert browser.wait_for(query_string_fixture.is_state_on_another_widget_now, 3)
+
+
+@with_fixtures(WebFixture, QueryStringFixture, ResponsiveDisclosureFixture)
+def test_refresh_widgets_with_widget_arguments(web_fixture, query_string_fixture, responsive_disclosure_fixture):
+    """Widget arguemnts are taken into account when widgets are refreshed by an input"""
+    fixture = responsive_disclosure_fixture
+
+    class MyForm(Form):
+        def __init__(self, view, an_object):
+            self.arg = 'default'
+            super().__init__(view, 'myform')
+            self.enable_refresh()
+            self.use_layout(FormLayout())
+            trigger_input = self.layout.add_input(SelectInput(self, an_object.fields.choice))
+
+            refreshed_widget = self.add_child(P(self.view, text='%s My state is now %s' % (self.arg, an_object.choice)))
+            refreshed_widget.set_id('refreshing-paragraph1')
+            refreshed_widget.enable_refresh()
+
+            trigger_input.set_refresh_widgets([refreshed_widget])
+
+            link = self.add_child(A(view, Url('#arg=changed'), description='Hash change link'))
+
+        query_fields = ExposedNames()
+        query_fields.arg = lambda i: Field()
+
+    fixture.MyForm = MyForm
+
+    wsgi_app = web_fixture.new_wsgi_app(enable_js=True, child_factory=fixture.MainWidget.factory())
+    web_fixture.reahl_server.set_app(wsgi_app)
+    browser = web_fixture.driver_browser
+    browser.open('/')
+
+    assert browser.wait_for(query_string_fixture.is_state_labelled_now, 'default My state',  1)
+    browser.select(XPath.select_labelled('Choice'), 'Three')
+    assert browser.wait_for(query_string_fixture.is_state_labelled_now, 'default My state',  3)
+
+    browser.click(XPath.link().with_text('Hash change link'))
+    assert browser.wait_for(query_string_fixture.is_state_labelled_now, 'changed My state',  3)
+    browser.select(XPath.select_labelled('Choice'), 'Two')
+    assert browser.wait_for(query_string_fixture.is_state_labelled_now, 'changed My state',  2)
 
 
 @with_fixtures(WebFixture, QueryStringFixture, ResponsiveDisclosureFixture)
@@ -302,12 +343,11 @@ def test_overridden_names(web_fixture, query_string_fixture, responsive_disclosu
         def __init__(self):
             self.choice = [1]
 
-        @exposed
-        def fields(self, fields):
-            fields.choice = MultiChoiceField([Choice(1, IntegerField(label='One')),
-                                                Choice(2, IntegerField(label='Two')),
-                                                Choice(3, IntegerField(label='Three'))],
-                                                label='Choice')
+        fields = ExposedNames()
+        fields.choice = lambda i: MultiChoiceField([Choice(1, IntegerField(label='One')),
+                                                    Choice(2, IntegerField(label='Two')),
+                                                    Choice(3, IntegerField(label='Three'))],
+                                                   label='Choice')
     fixture.ModelObject = ModelObject
 
     def create_trigger_input(form, an_object):
@@ -397,12 +437,11 @@ def test_form_values_are_not_persisted_until_form_is_submitted(web_fixture, resp
         id = Column(Integer, primary_key=True)
         choice = Column(Integer, default=1)
 
-        @exposed
-        def fields(self, fields):
-            fields.choice = ChoiceField([Choice(1, IntegerField(label='One')),
-                                         Choice(2, IntegerField(label='Two')),
-                                         Choice(3, IntegerField(label='Three'))],
-                                         label='Choice')
+        fields = ExposedNames()
+        fields.choice = lambda i: ChoiceField([Choice(1, IntegerField(label='One')),
+                                               Choice(2, IntegerField(label='Two')),
+                                               Choice(3, IntegerField(label='Three'))],
+                                              label='Choice')
     fixture.ModelObject = ModelObject
 
     class FormWithButton(fixture.MyForm):
@@ -412,9 +451,8 @@ def test_form_values_are_not_persisted_until_form_is_submitted(web_fixture, resp
             self.define_event_handler(self.events.submit)
             self.add_child(ButtonInput(self, self.events.submit))
 
-        @exposed
-        def events(self, events):
-            events.submit = Event(label='Submit')
+        events = ExposedNames()
+        events.submit = lambda i: Event(label='Submit')
 
     fixture.MyForm = FormWithButton
 
@@ -463,19 +501,17 @@ class DisclosedInputFixture(Fixture):
                 self.trigger_field = fixture.default_trigger_field_value
                 self.email = None
 
-            @exposed
-            def events(self, events):
-                events.an_event = Event(label='click me', action=Action(self.submit))
+            events = ExposedNames()
+            events.an_event = lambda i: Event(label='click me', action=Action(i.submit))
 
             def submit(self):
                 if fixture.raise_domain_exception_on_submit:
                     raise DomainException()
                 fixture.submitted_model_object = self
 
-            @exposed
-            def fields(self, fields):
-                fields.trigger_field = BooleanField(label='Trigger field')
-                fields.email = EmailField(required=True, label='Email')
+            fields = ExposedNames()
+            fields.trigger_field = lambda i: BooleanField(label='Trigger field')
+            fields.email = lambda i: EmailField(required=True, label='Email')
 
         class MyForm(Form):
             def __init__(self, view):
@@ -636,15 +672,13 @@ class TabOrderFixture(Fixture):
                 self.other = None
                 self.edge = None
 
-            @exposed
-            def events(self, events):
-                events.an_event = Event(label='click me')
+            events = ExposedNames()
+            events.an_event = lambda i: Event(label='click me')
 
-            @exposed
-            def fields(self, fields):
-                fields.trigger_field = BooleanField(label='Trigger field')
-                fields.other = Field(label='Other')
-                fields.edge = Field(label='Edge')
+            fields = ExposedNames()
+            fields.trigger_field = lambda i: BooleanField(label='Trigger field')
+            fields.other = lambda i: Field(label='Other')
+            fields.edge = lambda i: Field(label='Edge')
 
         class MyForm(Form):
             def __init__(self, view):
@@ -722,10 +756,9 @@ class TimingFixture(BlockingRefreshFixture):
             def __init__(self):
                 self.choice = 1
 
-            @exposed
-            def fields(self, fields):
-                fields.some_text = Field(label='Some Text')
-                fields.trigger_field = Field(label='Trigger')
+            fields = ExposedNames()
+            fields.some_text = lambda i: Field(label='Some Text')
+            fields.trigger_field = lambda i: Field(label='Trigger')
         return ModelObject
 
     def new_MyForm(self):
@@ -804,10 +837,9 @@ class NestedResponsiveDisclosureFixture(Fixture):
                 self.trigger_field = False
                 self.nested_trigger_field = False
 
-            @exposed
-            def fields(self, fields):
-                fields.trigger_field = BooleanField(label='Trigger field')
-                fields.nested_trigger_field = BooleanField(label='Nested trigger field')
+            fields = ExposedNames()
+            fields.trigger_field = lambda i: BooleanField(label='Trigger field')
+            fields.nested_trigger_field = lambda i: BooleanField(label='Nested trigger field')
 
         return ModelObject
 
@@ -930,9 +962,8 @@ def test_browser_back_after_state_changes_goes_to_previous_url(web_fixture, quer
             self.define_event_handler(self.events.submit)
             self.add_child(ButtonInput(self, self.events.submit))
 
-        @exposed
-        def events(self, events):
-            events.submit = Event(label='Submit')
+        events = ExposedNames()
+        events.submit = lambda i: Event(label='Submit')
 
     fixture.MyForm = FormWithButton
 
@@ -981,18 +1012,16 @@ class RecalculateWidgetFixture(Fixture):
             def submit(self):
                 raise DomainException(message='An exception happened on submit')
 
-            @exposed
-            def events(self, events):
-                events.choice_changed = Event(action=Action(self.recalculate))
-                events.submit = Event(action=Action(self.submit))
+            events = ExposedNames()
+            events.choice_changed = lambda i: Event(action=Action(i.recalculate))
+            events.submit = lambda i: Event(action=Action(i.submit))
 
-            @exposed
-            def fields(self, fields):
-                fields.choice = ChoiceField([Choice(1, IntegerField(label='One')),
-                                            Choice(2, IntegerField(label='Two')),
-                                            Choice(3, IntegerField(label='Three'))],
-                                            label='Choice')
-                fields.calculated_state = IntegerField(label='Calculated', writable=Allowed(not fixture.read_only))
+            fields = ExposedNames()
+            fields.choice = lambda i: ChoiceField([Choice(1, IntegerField(label='One')),
+                                                   Choice(2, IntegerField(label='Two')),
+                                                   Choice(3, IntegerField(label='Three'))],
+                                                  label='Choice')
+            fields.calculated_state = lambda i: IntegerField(label='Calculated', writable=Allowed(not fixture.read_only))
         return ModelObject
 
     def new_model_object(self):
@@ -1222,22 +1251,20 @@ def test_invalid_trigger_inputs(web_fixture, query_string_fixture, sql_alchemy_f
         def submit(self):
             raise DomainException(message='An exception happened on submit')
 
-        @exposed
-        def events(self, events):
-            events.choice_changed = Event(action=Action(self.recalculate))
-            events.submit = Event(action=Action(self.submit))
+        events = ExposedNames()
+        events.choice_changed = lambda i: Event(action=Action(i.recalculate))
+        events.submit = lambda i: Event(action=Action(i.submit))
 
-        @exposed
-        def fields(self, fields):
-            fields.choice = ChoiceField([Choice(1, IntegerField(label='One')),
-                                        Choice(2, IntegerField(label='Two')),
-                                        Choice(3, IntegerField(label='Three'))],
-                                        label='Choice')
-            fields.choice2 = ChoiceField([Choice(4, IntegerField(label='Four')),
-                                        Choice(5, IntegerField(label='Five')),
-                                        Choice(6, IntegerField(label='Six'))],
-                                        label='Choice2')
-            fields.calculated_state = IntegerField(label='Calculated', writable=Allowed(False))
+        fields = ExposedNames()
+        fields.choice = lambda i: ChoiceField([Choice(1, IntegerField(label='One')),
+                                               Choice(2, IntegerField(label='Two')),
+                                               Choice(3, IntegerField(label='Three'))],
+                                              label='Choice')
+        fields.choice2 = lambda i: ChoiceField([Choice(4, IntegerField(label='Four')),
+                                                Choice(5, IntegerField(label='Five')),
+                                                Choice(6, IntegerField(label='Six'))],
+                                               label='Choice2')
+        fields.calculated_state = lambda i: IntegerField(label='Calculated', writable=Allowed(False))
 
     class MyForm(Form):
         def __init__(self, view, an_object):
@@ -1310,22 +1337,20 @@ def test_invalid_non_trigger_input_corner_case(web_fixture, query_string_fixture
         def submit(self):
             raise DomainException(message='An exception happened on submit')
 
-        @exposed
-        def events(self, events):
-            events.choice_changed = Event(action=Action(self.recalculate))
-            events.submit = Event(action=Action(self.submit))
+        events = ExposedNames()
+        events.choice_changed = lambda i: Event(action=Action(i.recalculate))
+        events.submit = lambda i: Event(action=Action(i.submit))
 
-        @exposed
-        def fields(self, fields):
-            fields.choice = ChoiceField([Choice(1, IntegerField(label='One')),
-                                        Choice(2, IntegerField(label='Two')),
-                                        Choice(3, IntegerField(label='Three'))],
-                                        label='Choice')
-            fields.choice3 = ChoiceField([Choice(7, IntegerField(label='Seven')),
-                                        Choice(8, IntegerField(label='Eight')),
-                                        Choice(9, IntegerField(label='Nine'))],
-                                        label='Choice3')
-            fields.calculated_state = IntegerField(label='Calculated', writable=Allowed(False))
+        fields = ExposedNames()
+        fields.choice = lambda i: ChoiceField([Choice(1, IntegerField(label='One')),
+                                               Choice(2, IntegerField(label='Two')),
+                                               Choice(3, IntegerField(label='Three'))],
+                                              label='Choice')
+        fields.choice3 = lambda i: ChoiceField([Choice(7, IntegerField(label='Seven')),
+                                                Choice(8, IntegerField(label='Eight')),
+                                                Choice(9, IntegerField(label='Nine'))],
+                                               label='Choice3')
+        fields.calculated_state = lambda i: IntegerField(label='Calculated', writable=Allowed(False))
 
     class MyForm(Form):
         def __init__(self, view, an_object):
