@@ -1,4 +1,4 @@
-# Copyright 2013-2021 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2022 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -22,7 +22,7 @@ from reahl.component.exceptions import ProgrammerError
 
 from reahl.browsertools.browsertools import Browser, XPath
 
-from reahl.component.modelinterface import exposed, IntegerField, MultiChoiceField, Choice, Action, Event
+from reahl.component.modelinterface import ExposedNames, IntegerField, MultiChoiceField, Choice, Action, Event
 from reahl.web.fw import Bookmark, Widget
 from reahl.web.ui import A, P, Form, TextInput, Div, NestedForm, ButtonInput, FormLayout
 
@@ -82,9 +82,8 @@ def test_query_string_widget_arguments(web_fixture, value_scenarios):
             super().__init__(view)
             self.add_child(P(view, text=str(self.arg_directly_on_widget)))
 
-        @exposed
-        def query_fields(self, fields):
-            fields.arg_directly_on_widget = fixture.field.unbound_copy() # We use a copy, because the Field on the fixture is only constructed once, but we get here more than once during a GET
+        query_fields = ExposedNames()
+        query_fields.arg_directly_on_widget = lambda i: fixture.field.unbound_copy() # We use a copy, because the Field on the fixture is only constructed once, but we get here more than once during a GET
 
     wsgi_app = web_fixture.new_wsgi_app(enable_js=True, child_factory=WidgetWithQueryArguments.factory())
     browser = Browser(wsgi_app)
@@ -100,9 +99,8 @@ def test_query_string_prepopulates_form(web_fixture, value_scenarios):
     fixture = value_scenarios
 
     class ModelObject:
-        @exposed
-        def fields(self, fields):
-            fields.arg_on_other_object = fixture.field
+        fields = ExposedNames()
+        fields.arg_on_other_object = lambda i: fixture.field
 
     class FormWithQueryArguments(Form):
         def __init__(self, view):
@@ -111,9 +109,8 @@ def test_query_string_prepopulates_form(web_fixture, value_scenarios):
             self.use_layout(FormLayout())
             self.layout.add_input(TextInput(self, self.model_object.fields.arg_on_other_object))
 
-        @exposed
-        def query_fields(self, fields):
-            fields.arg_on_other_object = self.model_object.fields.arg_on_other_object
+        query_fields = ExposedNames()
+        query_fields.arg_on_other_object = lambda i: i.model_object.fields.arg_on_other_object
 
     wsgi_app = web_fixture.new_wsgi_app(enable_js=True, child_factory=FormWithQueryArguments.factory())
     browser = Browser(wsgi_app)
@@ -126,6 +123,8 @@ def test_query_string_prepopulates_form(web_fixture, value_scenarios):
 class QueryStringFixture(Fixture):
     def is_state_now(self, state):
         return self.is_state_labelled_now('My state', state)
+    def is_state_on_another_widget_now(self, state):
+        return self.is_state_labelled_now('Another widget state', state)
 
     def is_state_labelled_now(self, label, state):
         return self.web_fixture.driver_browser.is_element_present(XPath.paragraph().including_text('%s is now %s' % (label, state)))
@@ -147,9 +146,8 @@ def test_widgets_with_bookmarkable_state(web_fixture, query_string_fixture, valu
             self.add_child(P(self.view, text='My state is now %s' % self.fancy_state))
             fixture.widget = self
 
-        @exposed
-        def query_fields(self, fields):
-            fields.fancy_state = value_scenarios.field.unbound_copy()
+        query_fields = ExposedNames()
+        query_fields.fancy_state = lambda i: value_scenarios.field.unbound_copy()
 
     wsgi_app = web_fixture.new_wsgi_app(enable_js=True, child_factory=MyFancyWidget.factory())
     web_fixture.reahl_server.set_app(wsgi_app)
@@ -201,10 +199,9 @@ def test_refreshing_only_for_specific_args(web_fixture, query_string_fixture):
             self.add_child(P(self.view, text='My non-refreshing state is now %s' % self.non_refreshing_state))
             fixture.widget = self
 
-        @exposed
-        def query_fields(self, fields):
-            fields.refreshing_state = IntegerField(required=False, default=1)
-            fields.non_refreshing_state = IntegerField(required=False, default=2)
+        query_fields = ExposedNames()
+        query_fields.refreshing_state = lambda i: IntegerField(required=False, default=1)
+        query_fields.non_refreshing_state = lambda i: IntegerField(required=False, default=2)
 
     wsgi_app = web_fixture.new_wsgi_app(enable_js=True, child_factory=MyFancyWidget.factory())
     web_fixture.reahl_server.set_app(wsgi_app)
@@ -244,9 +241,8 @@ def test_coactive_widgets_are_refreshed_when_their_widgets_are(web_fixture, quer
         def coactive_widgets(self):
             return super().coactive_widgets + [self.coactive_div]
 
-        @exposed
-        def query_fields(self, fields):
-            fields.fancy_state = IntegerField(required=False, default=1)
+        query_fields = ExposedNames()
+        query_fields.fancy_state = lambda i: IntegerField(required=False, default=1)
 
 
     class MyFancyWidget(Widget):
@@ -302,13 +298,11 @@ def test_refresh_nested_forms(web_fixture, query_string_fixture):
         def submit(self):
             fixture.submitted = True
 
-        @exposed
-        def query_fields(self, fields):
-            fields.fancy_state = IntegerField(required=False, default=1)
+        query_fields = ExposedNames()
+        query_fields.fancy_state = lambda i: IntegerField(required=False, default=1)
 
-        @exposed
-        def events(self, events):
-            events.submit = Event(action=Action(self.submit))
+        events = ExposedNames()
+        events.submit = lambda i: Event(action=Action(i.submit))
 
     wsgi_app = web_fixture.new_wsgi_app(enable_js=True, child_factory=MyFancyWidget.factory())
     web_fixture.reahl_server.set_app(wsgi_app)
